//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file calculates CSR, CRR, and safety factor against liquefaction per the approach of Boulanger and Idriss (2014).
// The analysis is performed based on CPT data which must be in Imperial units : depth (ft), cone resistance and sleeve friction in (tsf) and water pressure in (psi).
// This file uses some variables and functions from "basicIntrep.cpp" and "basicIntrep.hpp" files.
//===========================================================================================================================

#include "B&I(2014).hpp"

B_I_2014::B_I_2014 () {
    //Do nothing
}

B_I_2014::~B_I_2014 () {
    //Do nothing
}

void B_I_2014::B_I_liqAaly () {

    readCPT ();
    basicCalcs ();

    double new_m_pow;
    double Msf_max;

    for (int i=0; i < numDataPoint; i++) {

    //Calculate Fines Content per Boulanger and Idriss (2014)
        if (80.0*Ic[i]-137.0 < 0) {
            FC.push_back(0.0);
        } else if (80.0*Ic[i]-137.0 >= 100) {
            FC.push_back(100.0);
        } else {
            FC.push_back(80.0*Ic[i]-137.0);
        }

    // iteration to calculate m and qc1N_cs per Boulanger and Idriss (2014)
        m_pow.push_back(0.782);
        new_m_pow =0.264;

        Cm.push_back(min(1.7, pow(1.058/insitu_EffStress[i],m_pow[i])));
        qc1N.push_back(Cm[i]*q_t[i]/1.058);
        d_qc1N.push_back((11.9+qc1N[i]/14.6)*exp(1.63-9.7/(FC[i]+2)-pow(15.7/(FC[i]+2),2)));
        qc1N_cs.push_back(qc1N[i]+d_qc1N[i]);

        replace( m_pow.begin(), m_pow.end(), m_pow[i], min (0.782, max(0.264,1.338-0.249*pow(min(254.0,max(21.0,qc1N_cs[i])),0.264))));

        while ( abs(m_pow[i]-new_m_pow) > 0.01) {

            replace( Cm.begin(), Cm.end(), Cm[i], min(1.7, pow(1.058/insitu_EffStress[i],m_pow[i])));
            replace( qc1N.begin(), qc1N.end(), qc1N[i], Cm[i]*q_t[i]/1.058);
            replace( d_qc1N.begin(), d_qc1N.end(), d_qc1N[i], (11.9+qc1N[i]/14.6)*exp(1.63-9.7/(FC[i]+2)-pow(15.7/(FC[i]+2),2)));
            replace( qc1N_cs.begin(), qc1N_cs.end(), qc1N_cs[i], qc1N[i]+d_qc1N[i]);

            new_m_pow = m_pow[i];

            replace( m_pow.begin(), m_pow.end(), m_pow[i], min (0.782, max(0.264,1.338-0.249*pow(min(254.0,max(21.0,qc1N_cs[i])),0.264))));
        }

    // calculate CRR_M(7.5)_1atm
        if ( Ic[i] > Ic_cutoff || qc1N_cs[i] > 180 || depth[i] < historWT ) {
            CRR.push_back(4);
        } else {
            CRR.push_back(exp(qc1N_cs[i]/113.0 + pow(qc1N_cs[i]/1000.,2) - pow(qc1N_cs[i]/140.0,3) + pow(qc1N_cs[i]/137.0,4) -2.80));
        }

    // calculate k_sigma and Msf
        k_sigma.push_back(min(1.1,1-min(1/(37.3-8.27*pow(qc1N_cs[i],0.264)),0.3)*log(insitu_EffStress[i]/1.058)));

        Msf_max = min(2.2,1.09+ pow(qc1N_cs[i]/180.0,3));
        M_sf.push_back(1+(Msf_max-1)*(8.64*exp(-Mw/4.0)-1.325));

    // calculate r_d and Msf
        rd.push_back(exp(-1.012-1.126*sin(depth[i]/3.28/11.73+5.133)+(0.106+0.118*sin(depth[i]/3.28/11.28+5.142))*Mw));

    // calculate CSR
        CSR.push_back(0.65*(TotStress[i]/design_EffStress[i])*PGA*rd[i]);

    // calculate Factor of Safety
        if (CRR[i]==4.0) {
            FoS.push_back(2.0);
        } else {
            FoS.push_back(CRR[i]*k_sigma[i]*M_sf[i]/CSR[i]);
        }


    }
    // writes results into the user-defined output file
     writeOutputs();

}
