//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file calculates CSR, CRR, and safety factor against liquefaction per the approach of Youd et al. (2001).
// The analysis is performed based on CPT data which must be in Imperial units : depth (ft), cone resistance and sleeve friction in (tsf) and water pressure in (psi).
// This file uses some variables and functions from "basicIntrep.cpp" and "basicIntrep.hpp" files.
//===========================================================================================================================

#include "Youd_et_al_2001.hpp"

//Default constructor function
youd_et_al_2001::youd_et_al_2001() {
    //do nothing
}

//Destructor function
youd_et_al_2001::~youd_et_al_2001() {
    //do nothing
}

void youd_et_al_2001::youd_liqAnaly () {
    
    readCPT ();
    basicCalcs ();
    
    for (int i=0; i < numDataPoint; i++) {
        
        //Calculate Fines Content per Robertson and Wride (1998)
        if (Ic[i] > 1.64 && Ic[i] < 2.36 ) {
            FC.push_back(5.0);
        } else if (Ic[i] < 3.25) {
            FC.push_back(1.75*pow(Ic[i],3.25)-3.7);
        } else {
            FC.push_back(100.0);
        }
        
        //Calculate fines content correction factor Kc
        if (Ic[i] <= 1.64) {
            K_c.push_back(1.0);
        } else if (Ic[i] <= 2.6) {
            K_c.push_back(-0.403*pow(Ic[i],4)+5.58*pow(Ic[i],3)-21.63*pow(Ic[i],2)+33.75*Ic[i]-17.88);
        } else if (Ic[i] > 1.60 && Ic[i] < 2.36 && f_s[i]/(q_t[i]-TotStress[i]) <=0.005) {
            K_c.push_back(1.0);
        } else {
            K_c.push_back(1.0);
        }
        
        qc1N.push_back(CN[i]*q_t[i]/1.058);
        qc1N_cs.push_back(qc1N[i]*K_c[i]);
        
        // calculate CRR_M(7.5)_1atm
        if ( Ic[i] > Ic_cutoff || qc1N_cs[i] > 160 || depth[i] < historWT ) {
            CRR.push_back(4);
        } else if (qc1N_cs[i] < 50) {
            CRR.push_back(0.833*qc1N_cs[i]/1000.0+0.05);
        } else {
            CRR.push_back(93.0*pow(qc1N_cs[i]/1000.0,3)+0.08);
        }
        
        // calculate k_sigma and Msf
        if (0.65*design_EffStress[i] > 1.0) {
        k_sigma.push_back(-0.0006*pow(0.65*design_EffStress[i],3)+0.0169*pow(0.65*design_EffStress[i],2)-0.1628*0.65*design_EffStress[i]+1.1389);
        } else {
        k_sigma.push_back(1.0);
        }
        
        const double Msf_2001 = pow(10.0, 2.24)/pow(Mw,2.56);
        
        // calculate r_d
        if (depth[i]/3.28 < 9.15) {
            rd.push_back(1.0-0.00765*depth[i]/3.28);
        } else {
            rd.push_back(1.174-0.0267*depth[i]/3.28);
        }
        
        // calculate CSR
        CSR.push_back(0.65*(TotStress[i]/design_EffStress[i])*PGA*rd[i]);
        
        // calculate Factor of Safety
        if (CRR[i]==4.0) {
            FoS.push_back(2.0);
        } else {
            FoS.push_back(CRR[i]*k_sigma[i]*Msf_2001/CSR[i]);
        }
        
        
    }
    // writes results into the user-defined output file    
    writeOutputs();
    
}
        

